﻿Imports System.Net
Imports System.Net.Sockets
Imports System.Threading
Imports System.Text
Imports System.IO

Public Class HTTPSessione

    ' Set WWW Percorso Radice.
    ReadOnly RootPath As String = Directory.GetCurrentDirectory() & "\WWWRoot\"

    Public Event InfoGenerali(ByVal DatiInviati As String)

    Private tcpListener As TcpListener
    Private clientSocket As Socket
    Private clientThread As Thread

    Public Sub New(ByVal tcpListener As System.Net.Sockets.TcpListener)

        Me.tcpListener = tcpListener

    End Sub

    Public Sub ProcessThread()

        While (True)
            Try
                clientSocket = tcpListener.AcceptSocket()

                ' Socket Information
                Dim clientInfo As IPEndPoint = CType(clientSocket.RemoteEndPoint, IPEndPoint)

                ' §§§ Info dal il Server §§§
                RaiseEvent InfoGenerali("Client: " + clientInfo.Address.ToString() + ":" + clientInfo.Port.ToString())

                ' Imposta thread per ogni connessione al browser web 
                clientThread = New Thread(New ThreadStart(AddressOf ProcessRequest)) With {
                    .Name = Date.Now.Minute & ":" & Date.Now.Millisecond
                }

                clientThread.Start()

                ' §§§ Info dal il Server §§§
                RaiseEvent InfoGenerali("ClientThread N° " & clientThread.Name)

            Catch ex As Exception
                'MessageBox.Show(ex.StackTrace.ToString())

                'If clientSocket.Connected Then
                '    clientSocket.Close()
                'End If

            End Try
        End While

    End Sub

    Public Function NomeThread()

        Return clientThread.Name

    End Function

    Protected Sub ProcessRequest()
        Dim recvBytes(1024) As Byte
        Dim htmlReq As String = Nothing
        Dim bytes As Int32

        Try
            ' Ricevo Richieste HTTP dal Web Browser
            bytes = clientSocket.Receive(recvBytes, 0, clientSocket.Available, SocketFlags.None)
            htmlReq = Encoding.ASCII.GetString(recvBytes, 0, bytes)

            If bytes = 0 Then
                Exit Sub
            End If

            ' §§§ Info dal il Server §§§
            RaiseEvent InfoGenerali("Richiesta → HTTP:  " & htmlReq)

            ' Set default page
            Dim defaultPage As String = "index.html"

            Dim strArray() As String
            Dim strRequest As String

            strArray = htmlReq.Trim.Split(" ")

            ' Determine the HTTP method (GET only)
            If strArray(0).Trim().ToUpper.Equals("GET") Then
                strRequest = strArray(1).Trim

                If (strRequest.StartsWith("/")) Then
                    strRequest = strRequest.Substring(1).Replace("?", "")
                End If

                If (strRequest.EndsWith("/") Or strRequest.Equals("")) Then
                    strRequest = strRequest & defaultPage
                End If

                strRequest = RootPath & strRequest

                ' §§§ Info dal il Server §§§
                RaiseEvent InfoGenerali("Richiesta: " & strRequest)
                SendHTMLResponse(strRequest)

            Else ' Not HTTP GET method

                strRequest = RootPath & "Error\" & "400.html"
                SendHTMLResponse(strRequest)

            End If

        Catch ex As Exception

            'If clientSocket.Connected Then
            '    clientSocket.Close()
            'End If
        End Try
    End Sub

    ' Send HTTP Response
    Private Sub SendHTMLResponse(ByVal httpRequest As String)
        Try
            ' Get the file content of HTTP Request 
            Dim streamReader As New IO.StreamReader(httpRequest)
            Dim strBuff As String = streamReader.ReadToEnd()
            streamReader.Close()
            streamReader = Nothing

            ' The content Length of HTTP Request
            Dim respByte() As Byte = Encoding.ASCII.GetBytes(strBuff)

            ' Misura corretta delle Immagini!
            Dim Size As Integer = 0

            If GetContentType(httpRequest) = "text/html" Then
                Size = respByte.Length
            ElseIf GetContentType(httpRequest) = "image/ico" Then
                Size = IO.File.ReadAllBytes(RootPath & httpRequest.Remove(0, httpRequest.LastIndexOf("\"))).Length
            Else
                Size = IO.File.ReadAllBytes(RootPath & httpRequest.Remove(0, httpRequest.LastIndexOf("\"))).Length
            End If

            ' Set HTML Header
            Dim htmlHeader As String =
                "HTTP/1.1 200 OK" & Convert.ToChar(Keys.Return) & Convert.ToChar(Keys.LineFeed) &
                "Server: FintoServer " & Convert.ToChar(Keys.Return) & Convert.ToChar(Keys.LineFeed) &
                "Content-Length: " & Size & Convert.ToChar(Keys.Return) & Convert.ToChar(Keys.LineFeed) &
                "Content-Type: " & GetContentType(httpRequest) & Convert.ToChar(Keys.Return) & Convert.ToChar(Keys.LineFeed) &
            Convert.ToChar(Keys.Return) & Convert.ToChar(Keys.LineFeed)

            ' §§§ Info dal il Server §§§
            RaiseEvent InfoGenerali("Risposta ← HTTP:  " & htmlHeader)

            ' The content Length of HTML Header
            Dim headerByte() As Byte = Encoding.ASCII.GetBytes(htmlHeader)

            ' §§§ Info dal il Server §§§
            RaiseEvent InfoGenerali("Risposta: " & httpRequest & Convert.ToChar(Keys.Return) & Convert.ToChar(Keys.LineFeed))

            ' Invia HTML Header al Web Browser
            clientSocket.Send(headerByte, 0, headerByte.Length, SocketFlags.None)

            ' Invia il Tipo di Contenuto chiesto al Web Browser
            Select Case GetContentType(httpRequest)
                Case "text/html"
                    clientSocket.Send(respByte, 0, respByte.Length, SocketFlags.None)
                Case "text/css"
                    clientSocket.Send(respByte, 0, respByte.Length, SocketFlags.None)
                Case "image/jpeg"
                    clientSocket.SendFile(RootPath & httpRequest.Remove(0, httpRequest.LastIndexOf("\")))
                Case "image/x-icon"
                    clientSocket.SendFile(RootPath & httpRequest.Remove(0, httpRequest.LastIndexOf("\")))
            End Select

            ' Close HTTP Socket connection
            clientSocket.Shutdown(SocketShutdown.Both)
            clientSocket.Close()

        Catch ex As Exception

            'If clientSocket.Connected Then
            '    clientSocket.Close()
            'End If

        End Try

    End Sub

    ' Get Content Type
    Private Function GetContentType(ByVal httpRequest As String) As String

        If (httpRequest.EndsWith("html")) Then
            Return "text/html"
        ElseIf (httpRequest.EndsWith("htm")) Then
            Return "text/html"
        ElseIf (httpRequest.EndsWith("txt")) Then
            Return "text/plain"
        ElseIf (httpRequest.EndsWith("css")) Then
            Return "text/css"
        ElseIf (httpRequest.EndsWith("ico")) Then
            Return "image/x-icon"
        ElseIf (httpRequest.EndsWith("gif")) Then
            Return "image/gif"
        ElseIf (httpRequest.EndsWith("jpg")) Then
            Return "image/jpeg"
        ElseIf (httpRequest.EndsWith("jpeg")) Then
            Return "image/jpeg"
        ElseIf (httpRequest.EndsWith("pdf")) Then
            Return "application/pdf"
        ElseIf (httpRequest.EndsWith("pdf")) Then
            Return "application/pdf"
        ElseIf (httpRequest.EndsWith("doc")) Then
            Return "application/msword"
        ElseIf (httpRequest.EndsWith("xls")) Then
            Return "application/vnd.ms-excel"
        ElseIf (httpRequest.EndsWith("ppt")) Then
            Return "application/vnd.ms-powerpoint"
        Else
            Return "text/plain"
        End If

    End Function

End Class
